import {
    AppBar,
    CssBaseline,
    IconButton,
    Theme,
    Toolbar,
    Typography,
    WithStyles,
    withStyles,
} from '@material-ui/core';
import { MuiThemeProvider } from '@material-ui/core/styles';
import { SvgIconProps } from '@material-ui/core/SvgIcon';
import {
    AuthorizationProvider,
    Loading,
    NavigationProvider,
    Notifications,
    withAuthorization,
} from './components/index';
import { LightbulbFull, LightbulbOutline, SqlStreamStore } from './icons';
import { JSONSchema7 } from 'json-schema';
import React, { ComponentType, createElement, PureComponent } from 'react';
import { connect, createState } from './reactive';
import { Observable as obs } from 'rxjs';
import { actions, store, Viewer } from './stream-store';
import themes from './themes';
import { AuthorizationProps, HalLink, HalLinks } from './types';
import { mediaTypes, preventDefault } from './utils';
import getDisplayName from './components/getDisplayName';

const getSelfAlias = (links: HalLinks) =>
    Object.keys(links)
        .flatMap(rel => links[rel])
        .filter(({ rel }) => rel && rel.indexOf('streamStore:') === 0)
        .filter(
            ({ href }) =>
                !!links.self.filter(link => link.href === href).length,
        )
        .map(({ rel }) => rel);

const self$ = store.hal$.links$
    .filter(links => !!links.self)
    .map(getSelfAlias)
    .filter(rel => !!rel)
    .map(([link]) => link);

interface SqlStreamStoreBrowserState {
    loading: boolean;
    mediaType: string;
    _links: HalLinks;
    self: HalLink;
    theme: Theme;
    forms: { [rel: string]: JSONSchema7 };
}
const state$ = createState<SqlStreamStoreBrowserState>(
    obs.merge(
        self$.map(self => ['self', () => self]),
        store.hal$.links$.map(links => ['_links', () => links]),
        store.hal$.forms$.map(forms => ['forms', () => forms]),
        store.hal$.loading$.map(loading => ['loading', () => loading]),
        store.mediaType$.map(mediaType => ['mediaType', () => mediaType]),
        themes.theme$.map(theme => ['theme', () => theme]),
    ),
    obs.of({
        _links: {},
        forms: {},
        loading: false,
        mediaType: mediaTypes.hal,
        self: {
            href: '',
        },
        theme: themes.defaultTheme,
    }),
);

const onNavigate = (link: HalLink, authorization: string | undefined) => {

    console.log('treker');

    link.href.indexOf('#') === -1 && actions.get.request.next({ link, headers: { authorization } });
}

const initialNavigation = ({ authorization }: AuthorizationProps) =>
    onNavigate(
        { href: 'http://localhost:5021/store/', type: mediaTypes.hal },
        authorization,
    );

type DidMount<T> = (props: T) => void;
type WillUnmount<T> = (props: T) => void;

const mount = <T extends object>(
    didMount: DidMount<T>,
    willUnmount?: WillUnmount<T>,
) => (WrappedComponent: ComponentType<T>) => {
    class Mount extends PureComponent<T> {
        componentDidMount() {
            didMount(this.props);
        }

        componentWillUnmount() {
            if (!willUnmount) {
                return;
            }
            willUnmount(this.props);
        }

        render() {
            return React.createElement<T>(WrappedComponent, this.props);
        }
    }

    (Mount as ComponentType<T>).displayName = getDisplayName(
        'Mount',
        WrappedComponent,
    );

    return Mount;
};

const lightbulbs: { [key: string]: ComponentType<SvgIconProps> } = {
    dark: LightbulbFull,
    light: LightbulbOutline,
};

const heroStyle = {
    themeToggle: {
        marginLeft: 'auto',
    },
};
interface HeroProps {
    theme: Theme;
}

const onThemeToggle = preventDefault(() => themes.actions.type.next(void 0));

const Hero: ComponentType<HeroProps> = withStyles(heroStyle)(
    ({ theme, classes }: HeroProps & WithStyles<typeof heroStyle>) => (
        <AppBar position={'static'}>
            <Toolbar>
                <SqlStreamStore color={'action'} />
                <Typography variant={'h6'} color={'inherit'}>
                    Sql Stream Store
                </Typography>
                <IconButton
                    onClick={onThemeToggle}
                    className={classes.themeToggle}
                >
                    {createElement(lightbulbs[theme.palette.type])}
                </IconButton>
            </Toolbar>
        </AppBar>
    ),
);

const SqlStreamStoreBrowser: ComponentType<
    SqlStreamStoreBrowserState
> = withAuthorization()(
    mount<SqlStreamStoreBrowserState & AuthorizationProps>(initialNavigation)(
        ({ loading, theme, ...props }) => (
            <MuiThemeProvider theme={theme}>
                <div>
                    <CssBaseline />
                    <Hero theme={theme} />
                    <Loading open={loading} />
                    <NavigationProvider onNavigate={onNavigate}>
                        <Viewer {...props} />
                        <Notifications />
                    </NavigationProvider>
                </div>
            </MuiThemeProvider>
        ),
    ),
);

export const AuthorizedSqlStreamStoreBrowser: ComponentType<
    SqlStreamStoreBrowserState & AuthorizationProps
> = ({ authorization, ...props }) => (
    <AuthorizationProvider authorization={authorization}>
        <SqlStreamStoreBrowser {...props} />
    </AuthorizationProvider>
);

export default connect<SqlStreamStoreBrowserState, AuthorizationProps>(state$)(
    AuthorizedSqlStreamStoreBrowser,
);